@@ -34,6 +34,7 @@ NSString *const VLCPlaybackServicePlaybackDidStop = @"VLCPlaybackServicePlayback
 NSString *const VLCPlaybackServicePlaybackMetadataDidChange = @"VLCPlaybackServicePlaybackMetadataDidChange";
 NSString *const VLCPlaybackServicePlaybackDidFail = @"VLCPlaybackServicePlaybackDidFail";
 NSString *const VLCPlaybackServicePlaybackPositionUpdated = @"VLCPlaybackServicePlaybackPositionUpdated";
+NSString *const VLCPlaybackServicePlaybackModeUpdated = @"VLCPlaybackServicePlaybackModeUpdated";
 
 #if TARGET_OS_IOS
 @interface VLCPlaybackService () <VLCMediaPlayerDelegate, VLCMediaDelegate, VLCMediaListPlayerDelegate, EqualizerViewDelegate>
@@ -60,7 +61,6 @@ NSString *const VLCPlaybackServicePlaybackPositionUpdated = @"VLCPlaybackService
 
     BOOL _mediaWasJustStarted;
     int _majorPositionChangeInProgress;
-    BOOL _recheckForExistingThumbnail;
     BOOL _externalAudioPlaybackDeviceConnected;
 
     NSLock *_playbackSessionManagementLock;
@@ -315,17 +315,22 @@ NSString *const VLCPlaybackServicePlaybackPositionUpdated = @"VLCPlaybackService
     }
     NSUserDefaults *userDefaults = [NSUserDefaults standardUserDefaults];
 
-    // Set last selected equalizer profile if enabled
-    _mediaPlayer.equalizerEnabled = ![userDefaults boolForKey:kVLCSettingEqualizerProfileDisabled];
+    BOOL equalizerEnabled = ![userDefaults boolForKey:kVLCSettingEqualizerProfileDisabled];
 
-    if (_mediaPlayer.equalizerEnabled) {
+    if (equalizerEnabled) {
+        NSArray *presets = [VLCAudioEqualizer presets];
         unsigned int profile = (unsigned int)[userDefaults integerForKey:kVLCSettingEqualizerProfile];
-        [_mediaPlayer resetEqualizerFromProfile:profile];
-        /* make sure the preset's preamp setting is deployed */
-        [_mediaPlayer setPreAmplification:[_mediaPlayer preAmplification]];
+        VLCAudioEqualizer *equalizer = [[VLCAudioEqualizer alloc] initWithPreset:presets[profile]];
+        equalizer.preAmplification = [userDefaults floatForKey:kVLCSettingDefaultPreampLevel];
+        _mediaPlayer.equalizer = equalizer;
     } else {
-        /* the default is 6 dB leading to audio compression */
-        [_mediaPlayer setPreAmplification:[userDefaults floatForKey:kVLCSettingDefaultPreampLevel]];
+        float preampValue = [userDefaults floatForKey:kVLCSettingDefaultPreampLevel];
+        if (preampValue != 0.) {
+            APLog(@"Enforcing presumbly disabled equalizer due to custom preamp value of %f2.0", preampValue);
+            VLCAudioEqualizer *equalizer = [[VLCAudioEqualizer alloc] init];
+            equalizer.preAmplification = preampValue;
+            _mediaPlayer.equalizer = equalizer;
+        }
     }
 
     _mediaWasJustStarted = YES;
@@ -400,13 +405,8 @@ NSString *const VLCPlaybackServicePlaybackPositionUpdated = @"VLCPlaybackService
             if (_mediaPlayer.state == VLCMediaPlayerStateStopped && _mediaPlayer.media != nil) {
                 // Since VLCMediaPlayerStateError is sometimes not matched with a valid media.
                 // This checks for decoded Audio & Video blocks.
-#if LIBVLC_VERSION_MAJOR == 3
-                finishedPlaybackWithError = (_mediaPlayer.media.numberOfDecodedAudioBlocks == 0)
-                                             && (_mediaPlayer.media.numberOfDecodedVideoBlocks == 0);
-#else
                 VLCMediaStats stats = _mediaPlayer.media.statistics;
                 finishedPlaybackWithError = (stats.decodedAudio == 0) && (stats.decodedVideo == 0);
-#endif
             } else {
                 finishedPlaybackWithError = _mediaPlayer.state == VLCMediaPlayerStateError;
             }
@@ -509,6 +509,7 @@ NSString *const VLCPlaybackServicePlaybackPositionUpdated = @"VLCPlaybackService
     if ([self.delegate respondsToSelector:@selector(playModeUpdated)]) {
         [self.delegate playModeUpdated];
     }
+    [[NSNotificationCenter defaultCenter] postNotificationName:VLCPlaybackServicePlaybackModeUpdated object:self];
 }
 
 - (BOOL)currentMediaHasChapters
@@ -599,6 +600,7 @@ NSString *const VLCPlaybackServicePlaybackPositionUpdated = @"VLCPlaybackService
     if ([self.delegate respondsToSelector:@selector(playModeUpdated)]) {
         [self.delegate playModeUpdated];
     }
+    [[NSNotificationCenter defaultCenter] postNotificationName:VLCPlaybackServicePlaybackModeUpdated object:self];
 }
 
 - (NSInteger)indexOfCurrentAudioTrack
@@ -841,6 +843,7 @@ NSString *const VLCPlaybackServicePlaybackPositionUpdated = @"VLCPlaybackService
     if ([self.delegate respondsToSelector:@selector(playModeUpdated)]) {
         [self.delegate playModeUpdated];
     }
+    [[NSNotificationCenter defaultCenter] postNotificationName:VLCPlaybackServicePlaybackModeUpdated object:self];
 }
 
 - (void)shuffleMediaList {
@@ -876,6 +879,7 @@ NSString *const VLCPlaybackServicePlaybackPositionUpdated = @"VLCPlaybackService
         if ([self.delegate respondsToSelector:@selector(playModeUpdated)]) {
             [self.delegate playModeUpdated];
         }
+        [[NSNotificationCenter defaultCenter] postNotificationName:VLCPlaybackServicePlaybackModeUpdated object:self];
     } else if (self.repeatMode == VLCRepeatCurrentItem && !isButtonPressed) {
         return _currentIndex;
     }
@@ -947,6 +951,7 @@ NSString *const VLCPlaybackServicePlaybackPositionUpdated = @"VLCPlaybackService
                 if ([self.delegate respondsToSelector:@selector(playModeUpdated)]) {
                     [self.delegate playModeUpdated];
                 }
+                [[NSNotificationCenter defaultCenter] postNotificationName:VLCPlaybackServicePlaybackModeUpdated object:self];
             }
 
             if(_currentIndex > 0) {
@@ -1191,65 +1196,106 @@ NSString *const VLCPlaybackServicePlaybackPositionUpdated = @"VLCPlaybackService
 
 - (void)setAmplification:(CGFloat)amplification forBand:(unsigned int)index
 {
-    if (!_mediaPlayer.equalizerEnabled) {
-        [_mediaPlayer setEqualizerEnabled:YES];
+    VLCAudioEqualizer *equalizer = _mediaPlayer.equalizer;
+    if (!equalizer) {
+        equalizer = [[VLCAudioEqualizer alloc] init];
+        _mediaPlayer.equalizer = equalizer;
     }
 
-    [_mediaPlayer setAmplification:amplification forBand:index];
-
-    // For some reason we have to apply again preamp to apply change
-    [_mediaPlayer setPreAmplification:[_mediaPlayer preAmplification]];
+    NSArray *bands = equalizer.bands;
+    if (index < bands.count) {
+        VLCAudioEqualizerBand *band = equalizer.bands[index];
+        band.amplification = amplification;
+    }
 }
 
 - (CGFloat)amplificationOfBand:(unsigned int)index
 {
-    return [_mediaPlayer amplificationOfBand:index];
+    VLCAudioEqualizer *equalizer = _mediaPlayer.equalizer;
+    if (!equalizer) {
+        equalizer = [[VLCAudioEqualizer alloc] init];
+        _mediaPlayer.equalizer = equalizer;
+    }
+
+    NSArray *bands = equalizer.bands;
+    if (index < bands.count) {
+        VLCAudioEqualizerBand *band = equalizer.bands[index];
+        return band.amplification;
+    }
+    return 0.;
 }
 
 - (NSArray *)equalizerProfiles
 {
-    return _mediaPlayer.equalizerProfiles;
+    return VLCAudioEqualizer.presets;
 }
 
 - (void)resetEqualizerFromProfile:(unsigned int)profile
 {
     NSUserDefaults *userDefaults = [NSUserDefaults standardUserDefaults];
     if (profile == 0) {
-        _mediaPlayer.equalizerEnabled = NO;
+        _mediaPlayer.equalizer = nil;
         [userDefaults setBool:YES forKey:kVLCSettingEqualizerProfileDisabled];
-        [_mediaPlayer setPreAmplification:[userDefaults floatForKey:kVLCSettingDefaultPreampLevel]];
+
+        float preampValue = [userDefaults floatForKey:kVLCSettingDefaultPreampLevel];
+        if (preampValue != 6.0) {
+            APLog(@"Enforcing presumbly disabled equalizer due to custom preamp value of %f2.0", preampValue);
+            VLCAudioEqualizer *eq = [[VLCAudioEqualizer alloc] init];
+            eq.preAmplification = preampValue;
+            _mediaPlayer.equalizer = eq;
+        }
         return;
     }
 
     [userDefaults setBool:NO forKey:kVLCSettingEqualizerProfileDisabled];
-    _mediaPlayer.equalizerEnabled = YES;
 
     unsigned int actualProfile = profile - 1;
     [userDefaults setInteger:actualProfile forKey:kVLCSettingEqualizerProfile];
-    [_mediaPlayer resetEqualizerFromProfile:actualProfile];
+
+    NSArray *presets = [VLCAudioEqualizer presets];
+    VLCAudioEqualizer *equalizer = [[VLCAudioEqualizer alloc] initWithPreset:presets[actualProfile]];
+    _mediaPlayer.equalizer = equalizer;
 }
 
 - (void)setPreAmplification:(CGFloat)preAmplification
 {
-    if (!_mediaPlayer.equalizerEnabled)
-        [_mediaPlayer setEqualizerEnabled:YES];
-
-    [_mediaPlayer setPreAmplification:preAmplification];
+    VLCAudioEqualizer *equalizer = _mediaPlayer.equalizer;
+    if (!equalizer) {
+        equalizer = [[VLCAudioEqualizer alloc] init];
+    }
+    equalizer.preAmplification = preAmplification;
+    _mediaPlayer.equalizer = equalizer;
 }
 
 - (CGFloat)preAmplification
 {
-    return [_mediaPlayer preAmplification];
+    VLCAudioEqualizer *equalizer = _mediaPlayer.equalizer;
+    if (equalizer) {
+        return equalizer.preAmplification;
+    }
+
+    return [[NSUserDefaults standardUserDefaults] floatForKey:kVLCSettingDefaultPreampLevel];
 }
 
 - (unsigned int)numberOfBands
 {
-    return [_mediaPlayer numberOfBands];
+    /* we need to alloc an equalizer here to get the number of bands to have a proper UI
+     * in case no equalizer was configured yet */
+    VLCAudioEqualizer *equalizer = _mediaPlayer.equalizer;
+    if (!equalizer) {
+        equalizer = [[VLCAudioEqualizer alloc] init];
+    }
+    return (unsigned int)equalizer.bands.count;
 }
 
 - (CGFloat)frequencyOfBandAtIndex:(unsigned int)index
 {
-    return [_mediaPlayer frequencyOfBandAtIndex:index];
+    VLCAudioEqualizer *equalizer = _mediaPlayer.equalizer;
+    if (!equalizer) {
+        equalizer = [[VLCAudioEqualizer alloc] init];
+    }
+    VLCAudioEqualizerBand *band = equalizer.bands[index];
+    return band.frequency;
 }
 
 - (unsigned int)selectedEqualizerProfile
@@ -1538,7 +1584,9 @@ NSString *const VLCPlaybackServicePlaybackPositionUpdated = @"VLCPlaybackService
 #if TARGET_OS_IOS
 - (void)savePlaybackState
 {
-    [[VLCAppCoordinator sharedInstance].mediaLibraryService savePlaybackStateFrom:self];
+    BOOL activePlaybackSession = self.isPlaying || _playerIsSetup;
+    if (activePlaybackSession)
+        [[VLCAppCoordinator sharedInstance].mediaLibraryService savePlaybackStateFrom:self];
 }
 #endif
 
