@@ -107,7 +107,7 @@ class VideoPlayerViewController: UIViewController {
                                          height: UIScreen.main.bounds.height)
 
     // MARK: - Private
-
+    private var systemBrightness: Double?
     // MARK: - 360
 
     private var fov: CGFloat = 0
@@ -297,7 +297,16 @@ class VideoPlayerViewController: UIViewController {
 
     private lazy var brightnessControlView: BrightnessControlView = {
         let vc = BrightnessControlView()
-        vc.updateIcon(level: brightnessControl.fetchAndGetDeviceValue())
+        let defaults = UserDefaults.standard
+        if defaults.bool(forKey: kVLCPlayerShouldRememberBrightness) {
+            if let brightness = defaults.value(forKey: KVLCPlayerBrightness) as? Float {
+                vc.updateIcon(level:brightness)
+            } else {
+                vc.updateIcon(level:brightnessControl.fetchAndGetDeviceValue())
+            }
+        } else {
+            vc.updateIcon(level:brightnessControl.fetchAndGetDeviceValue())
+        }
         vc.translatesAutoresizingMaskIntoConstraints = false
         vc.alpha = 0
         return vc
@@ -439,8 +448,15 @@ class VideoPlayerViewController: UIViewController {
         return downSwipeRecognizer
     }()
 
+    private lazy var minimizeGestureRecognizer: UIPanGestureRecognizer = {
+        let dismissGestureRecognizer = UIPanGestureRecognizer(target: self, action: #selector(handleMinimizeGesture(_:)))
+        return dismissGestureRecognizer
+    }()
+
     private var isGestureActive: Bool = false
 
+    private var viewTranslation: CGPoint = CGPoint(x: 0, y: 0)
+
     // MARK: - Popup Views
 
     private lazy var equalizerPopupView: PopupView = {
@@ -501,6 +517,9 @@ class VideoPlayerViewController: UIViewController {
         self.playerController = playerController
         super.init(nibName: nil, bundle: nil)
         self.playerController.delegate = self
+        systemBrightness = UIScreen.main.brightness
+        NotificationCenter.default.addObserver(self, selector: #selector(systemBrightnessChanged), name: UIApplication.didBecomeActiveNotification, object: nil)
+        self.mediaNavigationBar.addGestureRecognizer(minimizeGestureRecognizer)
     }
 
     required init?(coder: NSCoder) {
@@ -584,12 +603,21 @@ class VideoPlayerViewController: UIViewController {
 
         let setIconVisibility = playbackService.adjustFilter.isEnabled ? showIcon : hideIcon
         setIconVisibility(optionsNavigationBar.videoFiltersButton)
+
+        view.transform = .identity
     }
 
     override func viewDidAppear(_ animated: Bool) {
         super.viewDidAppear(animated)
         // _viewAppeared = YES;
-
+        
+        let defaults = UserDefaults.standard
+        if defaults.bool(forKey: kVLCPlayerShouldRememberBrightness) {
+            if let brightness = defaults.value(forKey: KVLCPlayerBrightness) as? CGFloat {
+                UIScreen.main.brightness = brightness
+            }
+        }
+        
         playbackService.recoverDisplayedMetadata()
         // [self resetVideoFiltersSliders];
 
@@ -644,6 +672,12 @@ class VideoPlayerViewController: UIViewController {
     override func viewDidDisappear(_ animated: Bool) {
         super.viewDidDisappear(animated)
         deviceMotion.stopDeviceMotion()
+        let defaults = UserDefaults.standard
+        if defaults.bool(forKey: kVLCPlayerShouldRememberBrightness) {
+            let currentBrightness = UIScreen.main.brightness
+            defaults.set(currentBrightness, forKey: KVLCPlayerBrightness)
+            UIScreen.main.brightness = systemBrightness!
+        }
     }
 
     override func viewDidLoad() {
@@ -656,6 +690,10 @@ class VideoPlayerViewController: UIViewController {
         setupRendererDiscoverer()
     }
 
+    @objc func systemBrightnessChanged() {
+        systemBrightness = UIScreen.main.brightness
+    }
+
     @objc func setupQueueViewController(qvc: QueueViewController) {
         queueViewController = qvc
         queueViewController?.delegate = self
@@ -1148,6 +1186,45 @@ extension VideoPlayerViewController {
             statusLabel.showStatusMessage(hudString)
         }
     }
+
+    @objc private func handleMinimizeGesture(_ sender: UIPanGestureRecognizer) {
+        switch sender.state {
+        case .changed:
+            viewTranslation = sender.translation(in: view)
+            if viewTranslation.y < 0 {
+                return
+            }
+
+            UIView.animate(withDuration: 0.5,
+                           delay: 0,
+                           usingSpringWithDamping: 0.7,
+                           initialSpringVelocity: 1,
+                           options: .curveEaseOut,
+                           animations: {
+                self.view.transform = CGAffineTransform(translationX: 0, y: self.viewTranslation.y)
+            })
+
+            break
+        case .ended:
+            let halfHeight = view.frame.height / 2
+            if viewTranslation.y < halfHeight {
+                UIView.animate(withDuration: 0.5,
+                               delay: 0,
+                               usingSpringWithDamping: 0.7,
+                               initialSpringVelocity: 1,
+                               options: .curveEaseOut,
+                               animations: {
+                    self.view.transform = .identity
+                })
+            } else {
+                delegate?.videoPlayerViewControllerDidMinimize(self)
+            }
+
+            break
+        default:
+            break
+        }
+    }
 }
 
 // MARK: - Private setups
